/*******************************************

 SYSFUNCS.H - The KINT Interpreter
	Copyright 2002 Everett Kaser
	All rights reserved.

	Do not copy or redistribute without the
	author's explicit written permission.
 *******************************************/

void SysFunc(BYTE *Kregs, WORD opcode);
void SysCleanup();
WORD FitFont(char *text, int w, int h, WORD fixedpitch, SIZE *fontsize);
void GetDefaultFont();

#define	SYS_FILESIZE	0x00
#define	SYS_FOPEN		0x01
#define	SYS_FEOF		0x02
#define	SYS_FREAD		0x03
#define	SYS_FWRITE		0x04
#define	SYS_FSEEK		0x05
#define	SYS_FCLOSE		0x06
#define	SYS_FFINDFIRST	0x07
#define	SYS_FFINDNEXT	0x08
#define	SYS_FFINDCLOSE	0x09
#define	SYS_DELETE		0x0A
#define	SYS_MAKESHARECOPY	0x0B
#define	SYS_PRINTSTART	0x0C
#define	SYS_PRINTBEGPAGE 0x0D
#define	SYS_PRINTENDPAGE 0x0E
#define	SYS_PRINTSTOP	0x0F

#define	SYS_ALLOCMEM	0x10
#define	SYS_FREEMEM		0x11
#define	SYS_TIMEREG		0x12
#define	SYS_TIMERSTR	0x13
#define	SYS_TIMEMILLI	0x14

#define	SYS_MEMCPY		0x16
#define	SYS_MEMMOVE		0x17
#define	SYS_STRCPY		0x18
#define	SYS_STRCMP		0x19
#define	SYS_STRICMP		0x1A
#define	SYS_STRLEN		0x1B
#define	SYS_MEMSET		0x1C
#define	SYS_MEMCMP		0x1D
#define	SYS_SETCURSOR	0x1E
#define	SYS_SETPGMTITLE	0x1F

#define	SYS_CREBM		0x20
#define	SYS_DELBM		0x21

#define	SYS_SIZEBM		0x23
#define	SYS_DRAWBM		0x24
#define	SYS_STRETCHBM	0x25
#define	SYS_DRAWMBM		0x26
#define	SYS_LOADBM		0x27
#define	SYS_LOADMBM		0x28

#define SYS_GETWINDOWPOS 0x2A
#define	SYS_SETWINDOWPOS 0x2B
#define	SYS_GETVIEWSIZE 0x2C

#define	SYS_GETSCREENSIZE 0x2E
#define	SYS_FLUSHSCREEN	0x2F

#define	SYS_RECTEDGED	0x30
#define	SYS_RECTSOLID	0x31
#define	SYS_RECTOUTLINE	0x32
#define	SYS_LINES		0x33
#define	SYS_SETPIXEL	0x34
#define	SYS_GETPIXEL	0x35

#define	SYS_GETTEXTSIZE	0x39
#define	SYS_GETFONT		0x3A
#define	SYS_RELFONT		0x3B
#define	SYS_BEGTEXT		0x3C
#define	SYS_TEXTSTYLE	0x3D
#define	SYS_TEXT		0x3E
#define	SYS_ENDTEXT		0x3F

#define	SYS_PLAYSOUND	0x40
#define	SYS_PLAYMUSIC	0x41
#define	SYS_STOPMUSIC	0x42
#define	SYS_ISMUSIC		0x43

#define	SYS_STARTTIMER	0x48
#define	SYS_STOPTIMER	0x49
#define	SYS_GETEOL		0x4A
#define	SYS_PRINTGETTEXTSIZE 0x4B
#define	SYS_PRINTTEXT	0x4C
#define	SYS_PRINTBM		0x4D
#define	SYS_PRINTLINES	0x4E

#define	SYS_SYSTEMMSG	0x4F

//************************************************************
//* DIB BITMAP SECTION
//************************************************************

typedef struct {
	HBITMAP		hBitmap;
	HBITMAP		hBitmapAnd;	// for use by masked bitmaps
	BYTE		*pBits;
	DIBSECTION	ds;
	int			iRShift[3];
	int			iLShift[3];
	void		*pPrev, *pNext;
} DIBS, *PDIBS;

// graphic macro definitions
#define	DibWidth(hdib)	(((PDIBS)(hdib))->ds.dsBm.bmWidth)
#define DibHeight(hdib)	(((PDIBS)(hdib))->ds.dsBm.bmHeight)
#define	DibBitCount(hdib) (((PDIBS)(hdib))->ds.dsBm.bmBitsPixel)
#define DibRowLength(hdib) (4*((DibWidth((hdib))*DibBitCount((hdib))+31)/32))
#define DibCompression(hdib) (((PDIBS)(hdib))->ds.dsBmih.biCompression)
#define DibInfoHeaderSize(hdib) ((DWORD)(((PDIBS)(hdib))->ds.dsBmih.biSize))
#define	DibMaskSize(hdib) ((DWORD)((((DIBS *)(hdib))->ds.dsBmih.biCompression==BI_BITFIELDS) ? (3*sizeof(DWORD)):0))
#define	DibColorSize(hdib) ((DWORD)(DibNumColors((hdib)) * sizeof(RGBQUAD)))
#define	DibInfoSize(hdib) ((DWORD)(DibInfoHeaderSize((hdib)) + DibMaskSize((hdib)) + DibColorSize((hdib))))
#define	DibInfoHeaderPtr(hdib) ((BITMAPINFOHEADER *)(&(((PDIBS)(hdib))->ds.dsBmih)))
#define	DibMaskPtr(hdib) ((DWORD *)(((DIBS *)(hdib))->ds.dsBitfields))
#define	DibBitsPtr(hdib) ((void *)(((PDIBS)(hdib))->pBits))

// graphic function prototypes
int DibNumColors(PDIBS hdib);
BOOL DibGetColor(PDIBS hdib, int index, RGBQUAD * prgb);
PDIBS DibCreateFromInfo(BITMAPINFO * pbmi);
BOOL DibDelete(PDIBS hdib);
PDIBS DibFileLoad(char *fname);
HBITMAP DibCopyToDdb(PDIBS hdib, HWND hwnd, HPALETTE hPalette);
PDIBS DibCreate(DWORD w, DWORD h);

typedef struct {
//	HBITMAP		orBM, andBM;
	WORD		w, h;
	COLORREF	Colors[16];	// the FIRST color is used as the transparent color for masking
	PDIBS		horDIB, handDIB;
} MBMdata;

typedef struct {
	HFONT	hFont;
	WORD	used;
} FONTARRAY;

typedef struct {
	COLORREF	BkColor;
	COLORREF	TextColor;
	int			BkMode;
	HFONT		Font;
} OLDMODES;
